package ch.fr.heia.isc.gl1;

// Import of the DefaultView classes
import ch.fr.hes.il.simulife2024.IView;
import ch.fr.hes.il.simulife2024.AndJuzPerView;
import ch.fr.hes.il.simulife2024.BugOliRosDefaultView;
import ch.fr.hes.il.simulife2024.JulMicRolView;
// Import of the model class
import ch.fr.hes.il.simulife2024.model.Board;
// Import of the simulator classes
import ch.fr.hes.id.simulife2024.Simulator;
import ch.fr.hes.id.simulife2024.SimulableElement;
import ch.fr.hes.id.simulife2024.simulation.VitroSimulator;
// Import of the elements classes
import ch.fr.hes.rs.simulife2024.elements.Engineer;
//import ch.fr.hes.rs.simulife2024.elements.SolarPannel;
// Import of the analysis classes
import ch.fr.hes.id.simulife2024.Analyzer;
import ch.fr.hes.id.simulife2024.analysis.GridAnalyzer;
import ch.fr.hes.id.simulife2024.analysis.PositionAnalyzer;
//import ch.fr.hes.id.simulife2024.analysis.CellularCollisionAnalyzer;
// Import of cockpit classes
import ch.epfl.general_libraries.clazzes.ParamName;
import ch.epfl.general_libraries.experiment_aut.Experiment;
import ch.epfl.general_libraries.experiment_aut.WrongExperimentException;
import ch.epfl.general_libraries.results.AbstractResultsDisplayer;
import ch.epfl.general_libraries.results.AbstractResultsManager;
import ch.epfl.general_libraries.results.DataPoint;
import ch.epfl.general_libraries.results.Execution;
// Import of the java.util classes
import java.util.ArrayList;
import java.util.Map;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Simulife experiment which will be shown on the GUI (it shows as unused but
 * that's not true).
 */
public final class SimulifeExperiment implements Experiment {
    /**
     * To log messages.
     */
    private static final Logger LOGGER =
            Logger.getLogger(SimulifeExperiment.class.getName());
    /**
     * Board size.
     */
    private final int boardSize;
    /**
     * Simulation steps.
     */
    private final int simulationSteps;
    /**
     * Number of initial elements.
     */
    private final int numElements;

    /**
     * Constructor.
     *
     * @param bS       given in the UI - size of the square board.
     * @param sS given in the UI - number of steps for the
     *                        simulation.
     * @param nE     given in the UI - number of element to place.
     */
    public SimulifeExperiment(
            @ParamName(name = "Board Size", default_ = "10") final int bS,
            @ParamName(name = "Simulation Steps", default_ = "10") final int sS,
            @ParamName(name = "Number of Elements", default_ = "10")
            final int nE) {
        this.boardSize = bS;
        this.simulationSteps = sS;
        this.numElements = nE;
    }

    @Override
    public void run(final AbstractResultsManager man,
                    final AbstractResultsDisplayer dis)
            throws WrongExperimentException {
        Execution exec = new Execution();
        DataPoint dp = new DataPoint();
        dp.addProperty("Board Size", String.valueOf(boardSize));
        dp.addProperty(
                "Simulation Steps",
                String.valueOf(simulationSteps)
        );
        dp.addProperty(
                "Number of Elements",
                String.valueOf(numElements)
        );

        Board board = new Board(boardSize, boardSize);
        Simulator mySimulator = new VitroSimulator();
        mySimulator.setSimulableBoard(board);

        ArrayList<IView> views = new ArrayList<>();
        views.add(new JulMicRolView());
        views.add(new BugOliRosDefaultView());
        views.add(new AndJuzPerView());

        ArrayList<Analyzer> analyzers = new ArrayList<>();
        analyzers.add(new GridAnalyzer());
        analyzers.add(new PositionAnalyzer());
//        analyzers.add(new CellularCollisionAnalyzer());

        for (IView view : views) {
            view.setDataSource(board);
        }

        ArrayList<SimulableElement> elements = new ArrayList<>();
        for (int i = 0; i < numElements; i++) {
            elements.add(new Engineer(i));
//            elements.add(new SolarPannel(i));
        }

        Random random = new Random();
        for (SimulableElement elt : elements) {
            elt.setSimulableBoard(board);
            board.addElement(
                    random.nextInt(boardSize),
                    random.nextInt(boardSize),
                    elt
            );
        }

        for (IView view : views) {
            view.updateView();
        }

        ArrayList<ArrayList<Map<String, String>>> simResult =
                mySimulator.run(simulationSteps);

        for (Analyzer analyzer : analyzers) {
            String analysisResult = analyzer.getAnalysis(simResult).toString();
            dp.addResultProperty(
                    analyzer.getClass().getSimpleName(),
                    analysisResult
            );
        }

        exec.addDataPoint(dp);
        man.addExecution(exec);

        LOGGER.log(Level.INFO, "\n{0}", dp);
    }
}
